@@ -1,5 +1,5 @@
 from typing import Optional, List, Union, Any, Dict, Literal
-from pydantic import BaseModel
+from openprotein.pydantic import BaseModel, root_validator
 
 from openprotein.base import APISession
 from openprotein.api.jobs import AsyncJobFuture
@@ -16,6 +16,29 @@ class SequenceDataset(BaseModel):
     sequences: List[str]
 
 
+class _Prediction(BaseModel):
+    """Prediction details."""
+
+    @root_validator(pre=True)
+    def extract_pred(cls, values):
+        p = values.pop("properties")
+        name = list(p.keys())[0]
+        ymu = p[name]["y_mu"]
+        yvar = p[name]["y_var"]
+        p["name"] = name
+        p["y_mu"] = ymu
+        p["y_var"] = yvar
+
+        values.update(p)
+        return values
+
+    model_id: str
+    model_name: str
+    y_mu: Optional[float] = None
+    y_var: Optional[float] = None
+    name: Optional[str]
+
+
 class Prediction(BaseModel):
     """Prediction details."""
 
@@ -35,6 +58,17 @@ class PredictJobBase(Job):
 class PredictJob(PredictJobBase):
     """Properties about predict job returned via API."""
 
+    @root_validator(pre=True)
+    def extract_pred(cls, values):
+        # Extracting 'predictions' and 'sequences' from the input values
+        v = values.pop("result")
+        preds = [i["predictions"] for i in v]
+        seqs = [i["sequence"] for i in v]
+        values["result"] = [
+            {"sequence": i, "predictions": p} for i, p in zip(seqs, preds)
+        ]
+        return values
+
     class SequencePrediction(BaseModel):
         """Sequence prediction."""
 
@@ -42,7 +76,7 @@ class PredictJob(PredictJobBase):
         predictions: List[Prediction] = []
 
     result: Optional[List[SequencePrediction]] = None
-    job_type: Literal[JobType.workflow_predict] = JobType.workflow_predict
+    job_type: str
 
 
 @register_job_type(JobType.worflow_predict_single_site)
@@ -128,9 +162,9 @@ def _create_predict_job(
 def create_predict_job(
     session: APISession,
     sequences: SequenceDataset,
-    train_job: Any,
+    train_job: Optional[Any] = None,
     model_ids: Optional[List[str]] = None,
-) -> PredictJob:
+) -> FutureBase:
     """
     Creates a predict job with a given set of sequences and a train job.
 
@@ -167,8 +201,9 @@ def create_predict_job(
         model_ids = [model_ids]
     endpoint = "v1/workflow/predict"
     payload = {"sequences": sequences.sequences}
+    train_job_id = train_job.id if train_job is not None else None
     return _create_predict_job(
-        session, endpoint, payload, model_ids=model_ids, train_job_id=train_job.id
+        session, endpoint, payload, model_ids=model_ids, train_job_id=train_job_id
     )
 
 
@@ -177,7 +212,7 @@ def create_predict_single_site(
     sequence: SequenceData,
     train_job: Any,
     model_ids: Optional[List[str]] = None,
-) -> PredictJob:
+) -> FutureBase:
     """
     Creates a predict job for single site mutants with a given sequence and a train job.
 
@@ -318,6 +353,7 @@ class PredictFutureMixin:
 
     session: APISession
     job: PredictJob
+    id: Optional[str] = None
 
     def get_results(
         self, page_size: Optional[int] = None, page_offset: Optional[int] = None
@@ -344,6 +380,7 @@ class PredictFutureMixin:
         HTTPError
             If the GET request does not succeed.
         """
+        assert self.id is not None
         if "single_site" in self.job.job_type:
             return get_single_site_prediction_results(
                 self.session, self.id, page_size, page_offset
@@ -352,7 +389,7 @@ class PredictFutureMixin:
             return get_prediction_results(self.session, self.id, page_size, page_offset)
 
 
-class PredictFuture(PredictFutureMixin, AsyncJobFuture, FutureBase):
+class PredictFuture(PredictFutureMixin, AsyncJobFuture, FutureBase):  # type: ignore
     """Future Job for manipulating results"""
 
     job_type = [JobType.workflow_predict, JobType.worflow_predict_single_site]
@@ -372,24 +409,29 @@ class PredictFuture(PredictFutureMixin, AsyncJobFuture, FutureBase):
         return self.job.job_id
 
     def _fmt_results(self, results):
-        properties = set(results[0].dict()["predictions"][0]["properties"].keys())
+        properties = set(
+            list(i["properties"].keys())[0] for i in results[0].dict()["predictions"]
+        )
         dict_results = {}
         for p in properties:
             dict_results[p] = {}
             for i, r in enumerate(results):
                 s = r.sequence
-                props = r.predictions[0].properties[p]
+                props = [i.properties[p] for i in r.predictions if p in i.properties][0]
                 dict_results[p][s] = {"mean": props["y_mu"], "variance": props["y_var"]}
+        dict_results
         return dict_results
 
     def _fmt_ssp_results(self, results):
-        properties = set(results[0].dict()["predictions"][0]["properties"].keys())
+        properties = set(
+            list(i["properties"].keys())[0] for i in results[0].dict()["predictions"]
+        )
         dict_results = {}
         for p in properties:
             dict_results[p] = {}
             for i, r in enumerate(results):
-                s = f"{r.position+1}{r.amino_acid}"
-                props = r.predictions[0].properties[p]
+                s = s = f"{r.position+1}{r.amino_acid}"
+                props = [i.properties[p] for i in r.predictions if p in i.properties][0]
                 dict_results[p][s] = {"mean": props["y_mu"], "variance": props["y_var"]}
         return dict_results
 
@@ -408,19 +450,21 @@ class PredictFuture(PredictFutureMixin, AsyncJobFuture, FutureBase):
         """
         step = self.page_size
 
-        results = []
+        results: List = []
         num_returned = step
         offset = 0
 
         while num_returned >= step:
             try:
                 response = self.get_results(page_offset=offset, page_size=step)
+                assert isinstance(response.result, list)
                 results += response.result
                 num_returned = len(response.result)
                 offset += num_returned
             except APIError as exc:
                 if verbose:
                     print(f"Failed to get results: {exc}")
+
         if self.job.job_type == JobType.workflow_predict:
             return self._fmt_results(results)
         else:
@@ -444,7 +488,7 @@ class PredictService:
     def create_predict_job(
         self,
         sequences: List,
-        train_job: Any,
+        train_job: Optional[Any] = None,
         model_ids: Optional[List[str]] = None,
     ) -> PredictFuture:
         """
@@ -475,17 +519,18 @@ class PredictService:
         APIError
             If the backend refuses the job (due to sequence length or invalid inputs)
         """
-        if train_job.assaymetadata is not None:
-            if train_job.assaymetadata.sequence_length is not None:
-                if any(
-                    [
-                        train_job.assaymetadata.sequence_length != len(s)
-                        for s in sequences
-                    ]
-                ):
-                    raise InvalidParameterError(
-                        f"Predict sequences length {len(sequences[0])}  != training assaydata ({train_job.assaymetadata.sequence_length})"
-                    )
+        if train_job is not None:
+            if train_job.assaymetadata is not None:
+                if train_job.assaymetadata.sequence_length is not None:
+                    if any(
+                        [
+                            train_job.assaymetadata.sequence_length != len(s)
+                            for s in sequences
+                        ]
+                    ):
+                        raise InvalidParameterError(
+                            f"Predict sequences length {len(sequences[0])}  != training assaydata ({train_job.assaymetadata.sequence_length})"
+                        )
         if not train_job.done():
             print(f"WARNING: training job has status {train_job.status}")
             # raise InvalidParameterError(
@@ -494,7 +539,7 @@ class PredictService:
 
         sequence_dataset = SequenceDataset(sequences=sequences)
         return create_predict_job(
-            self.session, sequence_dataset, train_job, model_ids=model_ids
+            self.session, sequence_dataset, train_job, model_ids=model_ids  # type: ignore
         )
 
     def create_predict_single_site(
@@ -546,5 +591,5 @@ class PredictService:
 
         sequence_dataset = SequenceData(sequence=sequence)
         return create_predict_single_site(
-            self.session, sequence_dataset, train_job, model_ids=model_ids
+            self.session, sequence_dataset, train_job, model_ids=model_ids  # type: ignore
         )
